Main View — rabbitChat 1.0.0 documentation

# Main View

This is the main view module which manages main tornado connections. This module provides request handlers for managing simple HTTP requests as well as Websocket requests.

Although the websocket requests are actually sockJs requests which follows the sockjs protcol, thus it provide interface to sockjs connection handlers behind the scene.

## IndexHandler

class `rabbitChat.apps.main.views.` `IndexHandler`(application, request, **kwargs)

  This handler is a basic regular HTTP handler to serve the chatroom page.

  `get`()

    This method is called when a client does a simple GET request, all other HTTP requests like POST, PUT, DELETE, etc are ignored.

    Returns: | Returns the rendered main requested page, in this case its the chat page, index.html

  `SUPPORTED_METHODS` = ('GET', 'HEAD', 'POST', 'DELETE', 'PATCH', 'PUT', 'OPTIONS')
  `_ARG_DEFAULT` = []
  `_INVALID_HEADER_CHAR_RE` = <_sre.SRE_Pattern object>
  `__class__`

    alias of `type`

  `__delattr__`

    x.__delattr__(‘name’) <==> del x.name

  `__dict__` = dict_proxy({'__module__': 'rabbitChat.apps.main.views', '__doc__': 'This handler is a basic regular HTTP handler to serve the chatroom page.\n\n ', 'get': <function get at 0x110f05398>})
  `__format__`()

    default object formatter

  `__getattribute__`

    x.__getattribute__(‘name’) <==> x.name

  `__hash__`
  `__init__`(application, request, **kwargs)
  `__module__` = 'rabbitChat.apps.main.views'
  `__new__`(S, ...) → a new object with type S, a subtype of T
  `__reduce__`()

    helper for pickle

  `__reduce_ex__`()

    helper for pickle

  `__repr__`
  `__setattr__`

    x.__setattr__(‘name’, value) <==> x.name = value

  `__sizeof__`() → int

    size of object in memory, in bytes

  `__str__`
  `__subclasshook__`()

    Abstract classes can override this to customize issubclass().

    This is invoked early on by abc.ABCMeta.__subclasscheck__(). It should return True, False or NotImplemented. If it returns NotImplemented, the normal algorithm is used. Otherwise, it overrides the normal algorithm (and the outcome is cached).

  `__weakref__`

    list of weak references to the object (if defined)

  `_clear_headers_for_304`()
  `_convert_header_value`(value)
  `_decode_xsrf_token`(cookie)

    Convert a cookie string into a the tuple form returned by _get_raw_xsrf_token.

  `_execute`(*args, **kwargs)

    Executes this request with the given output transforms.

  `_get_argument`(name, default, source, strip=True)
  `_get_arguments`(name, source, strip=True)
  `_get_raw_xsrf_token`()

    Read or generate the xsrf token in its raw form.

    The raw_xsrf_token is a tuple containing:

    - version: the version of the cookie from which this token was read, or None if we generated a new token in this request.
    - token: the raw token data; random (non-ascii) bytes.
    - timestamp: the time this token was generated (will not be accurate for version 1 cookies)
  `_handle_request_exception`(e)
  `_log`()

    Logs the current request.

    Sort of deprecated since this functionality was moved to the Application, but left in place for the benefit of existing apps that have overridden this method.

  `_remove_control_chars_regex` = <_sre.SRE_Pattern object>
  `_request_summary`()
  `_stack_context_handle_exception`(type, value, traceback)
  `_template_loader_lock` = <thread.lock object>
  `_template_loaders` = {}
  `_ui_method`(method)
  `_ui_module`(name, module)
  `add_header`(name, value)

    Adds the given response header and value.

    Unlike set_header, add_header may be called multiple times to return multiple values for the same header.

  `check_etag_header`()

    Checks the `Etag` header against requests’s `If-None-Match`.

    Returns `True` if the request’s Etag matches and a 304 should be returned. For example:

        self.set_etag_header()
        if self.check_etag_header():
            self.set_status(304)
            return

    This method is called automatically when the request is finished, but may be called earlier for applications that override compute_etag and want to do an early check for `If-None-Match` before completing the request. The `Etag` header should be set (perhaps with set_etag_header) before calling this method.

  `check_xsrf_cookie`()

    Verifies that the `_xsrf` cookie matches the `_xsrf` argument.

    To prevent cross-site request forgery, we set an `_xsrf` cookie and include the same value as a non-cookie field with all `POST` requests. If the two do not match, we reject the form submission as a potential forgery.

    The `_xsrf` value may be set as either a form field named `_xsrf` or in a custom HTTP header named `X-XSRFToken` or `X-CSRFToken` (the latter is accepted for compatibility with Django).

    See http://en.wikipedia.org/wiki/Cross-site_request_forgery

    Prior to release 1.1.1, this check was ignored if the HTTP header `X-Requested-With: XMLHTTPRequest` was present. This exception has been shown to be insecure and has been removed. For more information please see http://www.djangoproject.com/weblog/2011/feb/08/security/ http://weblog.rubyonrails.org/2011/2/8/csrf-protection-bypass-in-ruby-on-rails

    Changed in version 3.2.2: Added support for cookie version 2. Both versions 1 and 2 are supported.

  `clear`()

    Resets all headers and content for this response.

  `clear_all_cookies`(path='/', domain=None)

    Deletes all the cookies the user sent with this request.

    See clear_cookie for more information on the path and domain parameters.

    Changed in version 3.2: Added the `path` and `domain` parameters.

  `clear_cookie`(name, path='/', domain=None)

    Deletes the cookie with the given name.

    Due to limitations of the cookie protocol, you must pass the same path and domain to clear a cookie as were used when that cookie was set (but there is no way to find out on the server side which values were used for a given cookie).

  `clear_header`(name)

    Clears an outgoing header, undoing a previous set_header call.

    Note that this method does not apply to multi-valued headers set by add_header.

  `compute_etag`()

    Computes the etag header to be used for this request.

    By default uses a hash of the content written so far.

    May be overridden to provide custom etag implementations, or may return None to disable tornado’s default etag support.

  `cookies`

    An alias for self.request.cookies <.httputil.HTTPServerRequest.cookies>.

  `create_signed_value`(name, value, version=None)

    Signs and timestamps a string so it cannot be forged.

    Normally used via set_secure_cookie, but provided as a separate method for non-cookie uses. To decode a value not stored as a cookie use the optional value argument to get_secure_cookie.

    Changed in version 3.2.1: Added the `version` argument. Introduced cookie version 2 and made it the default.

  `create_template_loader`(template_path)

    Returns a new template loader for the given path.

    May be overridden by subclasses. By default returns a directory-based loader on the given path, using the `autoescape` and `template_whitespace` application settings. If a `template_loader` application setting is supplied, uses that instead.

  `current_user`

    The authenticated user for this request.

    This is set in one of two ways:

    - A subclass may override get_current_user(), which will be called automatically the first time `self.current_user` is accessed. get_current_user() will only be called once per request, and is cached for future access:

          def get_current_user(self):
              user_cookie = self.get_secure_cookie("user")
                  if user_cookie:
                      return json.loads(user_cookie)
              return None

    - It may be set as a normal variable, typically from an overridden prepare():

          @gen.coroutine
          def prepare(self):
              user_id_cookie = self.get_secure_cookie("user_id")
              if user_id_cookie:
                  self.current_user = yield load_user(user_id_cookie)

    Note that prepare() may be a coroutine while get_current_user() may not, so the latter form is necessary if loading the user requires asynchronous operations.

    The user object may any type of the application’s choosing.

  `data_received`(chunk)

    Implement this method to handle streamed request data.

    Requires the .stream_request_body decorator.

  `decode_argument`(value, name=None)

    Decodes an argument from the request.

    The argument has been percent-decoded and is now a byte string. By default, this method decodes the argument as utf-8 and returns a unicode string, but this may be overridden in subclasses.

    This method is used as a filter for both get_argument() and for values extracted from the url and passed to get()/post()/etc.

    The name of the argument is provided if known, but may be None (e.g. for unnamed groups in the url regex).

  `delete`(*args, **kwargs)
  `finish`(chunk=None)

    Finishes this response, ending the HTTP request.

  `flush`(include_footers=False, callback=None)

    Flushes the current output buffer to the network.

    The `callback` argument, if given, can be used for flow control: it will be run when all flushed data has been written to the socket. Note that only one flush callback can be outstanding at a time; if another flush occurs before the previous flush’s callback has been run, the previous callback will be discarded.

    Changed in version 4.0: Now returns a .Future if no callback is given.

  `get_argument`(name, default=[], strip=True)

    Returns the value of the argument with the given name.

    If default is not provided, the argument is considered to be required, and we raise a MissingArgumentError if it is missing.

    If the argument appears in the url more than once, we return the last value.

    The returned value is always unicode.

  `get_arguments`(name, strip=True)

    Returns a list of the arguments with the given name.

    If the argument is not present, returns an empty list.

    The returned values are always unicode.

  `get_body_argument`(name, default=[], strip=True)

    Returns the value of the argument with the given name from the request body.

    If default is not provided, the argument is considered to be required, and we raise a MissingArgumentError if it is missing.

    If the argument appears in the url more than once, we return the last value.

    The returned value is always unicode.

    New in version 3.2.

  `get_body_arguments`(name, strip=True)

    Returns a list of the body arguments with the given name.

    If the argument is not present, returns an empty list.

    The returned values are always unicode.

    New in version 3.2.

  `get_browser_locale`(default='en_US')

    Determines the user’s locale from `Accept-Language` header.

    See http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.4

  `get_cookie`(name, default=None)

    Gets the value of the cookie with the given name, else default.

  `get_current_user`()

    Override to determine the current user from, e.g., a cookie.

    This method may not be a coroutine.

  `get_login_url`()

    Override to customize the login URL based on the request.

    By default, we use the `login_url` application setting.

  `get_query_argument`(name, default=[], strip=True)

    Returns the value of the argument with the given name from the request query string.

    If default is not provided, the argument is considered to be required, and we raise a MissingArgumentError if it is missing.

    If the argument appears in the url more than once, we return the last value.

    The returned value is always unicode.

    New in version 3.2.

  `get_query_arguments`(name, strip=True)

    Returns a list of the query arguments with the given name.

    If the argument is not present, returns an empty list.

    The returned values are always unicode.

    New in version 3.2.

  `get_secure_cookie`(name, value=None, max_age_days=31, min_version=None)

    Returns the given signed cookie if it validates, or None.

    The decoded cookie value is returned as a byte string (unlike get_cookie).

    Changed in version 3.2.1: Added the `min_version` argument. Introduced cookie version 2; both versions 1 and 2 are accepted by default.

  `get_secure_cookie_key_version`(name, value=None)

    Returns the signing key version of the secure cookie.

    The version is returned as int.

  `get_status`()

    Returns the status code for our response.

  `get_template_namespace`()

    Returns a dictionary to be used as the default template namespace.

    May be overridden by subclasses to add or modify values.

    The results of this method will be combined with additional defaults in the tornado.template module and keyword arguments to render or render_string.

  `get_template_path`()

    Override to customize template path for each handler.

    By default, we use the `template_path` application setting. Return None to load templates relative to the calling file.

  `get_user_locale`()

    Override to determine the locale from the authenticated user.

    If None is returned, we fall back to get_browser_locale().

    This method should return a tornado.locale.Locale object, most likely obtained via a call like `tornado.locale.get("en")`

  `head`(*args, **kwargs)
  `initialize`()

    Hook for subclass initialization.

    A dictionary passed as the third argument of a url spec will be supplied as keyword arguments to initialize().

    Example:

        class ProfileHandler(RequestHandler):
            def initialize(self, database):
                self.database = database

            def get(self, username):
                ...

        app = Application([
            (r'/user/(.*)', ProfileHandler, dict(database=database)),
            ])

  `locale`

    The locale for the current session.

    Determined by either get_user_locale, which you can override to set the locale based on, e.g., a user preference stored in a database, or get_browser_locale, which uses the `Accept-Language` header.

  `log_exception`(typ, value, tb)

    Override to customize logging of uncaught exceptions.

    By default logs instances of HTTPError as warnings without stack traces (on the `tornado.general` logger), and all other exceptions as errors with stack traces (on the `tornado.application` logger).

    New in version 3.1.

  `on_connection_close`()

    Called in async handlers if the client closed the connection.

    Override this to clean up resources associated with long-lived connections. Note that this method is called only if the connection was closed during asynchronous processing; if you need to do cleanup after every request override on_finish instead.

    Proxies may keep a connection open for a time (perhaps indefinitely) after the client has gone away, so this method may not be called promptly after the end user closes their connection.

  `on_finish`()

    Called after the end of a request.

    Override this method to perform cleanup, logging, etc. This method is a counterpart to prepare. `on_finish` may not produce any output, as it is called after the response has been sent to the client.

  `options`(*args, **kwargs)
  `patch`(*args, **kwargs)
  `post`(*args, **kwargs)
  `prepare`()

    Called at the beginning of a request before get/post/etc.

    Override this method to perform common initialization regardless of the request method.

    Asynchronous support: Decorate this method with .gen.coroutine or .return_future to make it asynchronous (the asynchronous decorator cannot be used on prepare). If this method returns a .Future execution will not proceed until the .Future is done.

    New in version 3.1: Asynchronous support.

  `put`(*args, **kwargs)
  `redirect`(url, permanent=False, status=None)

    Sends a redirect to the given (optionally relative) URL.

    If the `status` argument is specified, that value is used as the HTTP status code; otherwise either 301 (permanent) or 302 (temporary) is chosen based on the `permanent` argument. The default is 302 (temporary).

  `render`(template_name, **kwargs)

    Renders the template with the given arguments as the response.

  `render_string`(template_name, **kwargs)

    Generate the given template with the given arguments.

    We return the generated byte string (in utf8). To generate and write a template as a response, use render() above.

  `require_setting`(name, feature='this feature')

    Raises an exception if the given app setting is not defined.

  `reverse_url`(name, *args)

    Alias for Application.reverse_url.

  `send_error`(status_code=500, **kwargs)

    Sends the given HTTP error code to the browser.

    If flush() has already been called, it is not possible to send an error, so this method will simply terminate the response. If output has been written but not yet flushed, it will be discarded and replaced with the error page.

    Override write_error() to customize the error page that is returned. Additional keyword arguments are passed through to write_error.

  `set_cookie`(name, value, domain=None, expires=None, path='/', expires_days=None, **kwargs)

    Sets the given cookie name/value with the given options.

    Additional keyword arguments are set on the Cookie.Morsel directly. See http://docs.python.org/library/cookie.html#morsel-objects for available attributes.

  `set_default_headers`()

    Override this to set HTTP headers at the beginning of the request.

    For example, this is the place to set a custom `Server` header. Note that setting such headers in the normal flow of request processing may not do what you want, since headers may be reset during error handling.

  `set_etag_header`()

    Sets the response’s Etag header using `self.compute_etag()`.

    Note: no header will be set if `compute_etag()` returns `None`.

    This method is called automatically when the request is finished.

  `set_header`(name, value)

    Sets the given response header name and value.

    If a datetime is given, we automatically format it according to the HTTP specification. If the value is not a string, we convert it to a string. All header values are then encoded as UTF-8.

  `set_secure_cookie`(name, value, expires_days=30, version=None, **kwargs)

    Signs and timestamps a cookie so it cannot be forged.

    You must specify the `cookie_secret` setting in your Application to use this method. It should be a long, random sequence of bytes to be used as the HMAC secret for the signature.

    To read a cookie set with this method, use get_secure_cookie().

    Note that the `expires_days` parameter sets the lifetime of the cookie in the browser, but is independent of the `max_age_days` parameter to get_secure_cookie.

    Secure cookies may contain arbitrary byte values, not just unicode strings (unlike regular cookies)

    Changed in version 3.2.1: Added the `version` argument. Introduced cookie version 2 and made it the default.

  `set_status`(status_code, reason=None)

    Sets the status code for our response.

    Parameters: | status_code (int) – Response status code. If `reason` is `None`, it must be present in httplib.responses <http.client.responses>. reason (string) – Human-readable reason phrase describing the status code. If `None`, it will be filled in from httplib.responses <http.client.responses>.

  `settings`

    An alias for self.application.settings <Application.settings>.

  `static_url`(path, include_host=None, **kwargs)

    Returns a static URL for the given relative static file path.

    This method requires you set the `static_path` setting in your application (which specifies the root directory of your static files).

    This method returns a versioned url (by default appending `?v=<signature>`), which allows the static files to be cached indefinitely. This can be disabled by passing `include_version=False` (in the default implementation; other static file implementations are not required to support this, but they may support other options).

    By default this method returns URLs relative to the current host, but if `include_host` is true the URL returned will be absolute. If this handler has an `include_host` attribute, that value will be used as the default for all static_url calls that do not pass `include_host` as a keyword argument.

  `write`(chunk)

    Writes the given chunk to the output buffer.

    To write the output to the network, use the flush() method below.

    If the given chunk is a dictionary, we write it as JSON and set the Content-Type of the response to be `application/json`. (if you want to send JSON as a different `Content-Type`, call set_header after calling write()).

    Note that lists are not converted to JSON because of a potential cross-site security vulnerability. All JSON output should be wrapped in a dictionary. More details at http://haacked.com/archive/2009/06/25/json-hijacking.aspx/ and https://github.com/facebook/tornado/issues/1009

  `write_error`(status_code, **kwargs)

    Override to implement custom error pages.

    `write_error` may call write, render, set_header, etc to produce output as usual.

    If this error was caused by an uncaught exception (including HTTPError), an `exc_info` triple will be available as `kwargs["exc_info"]`. Note that this exception may not be the “current” exception for purposes of methods like `sys.exc_info()` or `traceback.format_exc`.

  `xsrf_form_html`()

    An HTML `<input/>` element to be included with all POST forms.

    It defines the `_xsrf` input value, which we check on all POST requests to prevent cross-site request forgery. If you have set the `xsrf_cookies` application setting, you must include this HTML within all of your HTML forms.

    In a template, this method should be called with `{% module xsrf_form_html() %}`

    See check_xsrf_cookie() above for more information.

  `xsrf_token`

    The XSRF-prevention token for the current user/session.

    To prevent cross-site request forgery, we set an ‘_xsrf’ cookie and include the same ‘_xsrf’ value as an argument with all POST requests. If the two do not match, we reject the form submission as a potential forgery.

    See http://en.wikipedia.org/wiki/Cross-site_request_forgery

    Changed in version 3.2.2: The xsrf token will now be have a random mask applied in every request, which makes it safe to include the token in pages that are compressed. See http://breachattack.com for more information on the issue fixed by this change. Old (version 1) cookies will be converted to version 2 when this method is called unless the `xsrf_cookie_version` Application setting is set to 1.

    Changed in version 4.3: The `xsrf_cookie_kwargs` Application setting may be used to supply additional cookie options (which will be passed directly to set_cookie). For example, `xsrf_cookie_kwargs=dict(httponly=True, secure=True)` will set the `secure` and `httponly` flags on the `_xsrf` cookie.

## ChatWebsocketHandler

class `rabbitChat.apps.main.views.` `ChatWebsocketHandler`(session)

  Websocket Handler implementing the sockjs Connection Class which will handle the websocket/sockjs connections.

  `on_open`(info)

    This method is called when a websocket/sockjs connection is opened for the first time.

    :param self The object :param info The information

    Returns: | It returns the websocket object

  `on_message`(message)

    This method is called when a message is received via the websocket/sockjs connection created initially.

    :param self The object :param message The message received via the connection.

    Returns: | Returns the published message back to all other subscribers.

  `on_close`()

    This method is called when a websocket/sockjs connection is closed.

    :param self The object

    Returns: | Doesn’t return anything, except a confirmation of closed connection back to web app.

  `genid`()
  `__class__`

    alias of `type`

  `__delattr__`

    x.__delattr__(‘name’) <==> del x.name

  `__dict__` = dict_proxy({'__module__': 'rabbitChat.apps.main.views', 'on_message': <function on_message at 0x110f05488>, 'genid': <function genid at 0x110f05578>, 'on_open': <function on_open at 0x110f05410>, 'on_close': <function on_close at 0x110f05500>, '__doc__': ' Websocket Handler implementing the sockjs Connection Class which will\n handle the websocket/sockjs connections.\n '})
  `__format__`()

    default object formatter

  `__getattribute__`

    x.__getattribute__(‘name’) <==> x.name

  `__hash__`
  `__init__`(session)

    Connection constructor.

    session
      Associated session
  `__module__` = 'rabbitChat.apps.main.views'
  `__new__`(S, ...) → a new object with type S, a subtype of T
  `__reduce__`()

    helper for pickle

  `__reduce_ex__`()

    helper for pickle

  `__repr__`
  `__setattr__`

    x.__setattr__(‘name’, value) <==> x.name = value

  `__sizeof__`() → int

    size of object in memory, in bytes

  `__str__`
  `__subclasshook__`()

    Abstract classes can override this to customize issubclass().

    This is invoked early on by abc.ABCMeta.__subclasscheck__(). It should return True, False or NotImplemented. If it returns NotImplemented, the normal algorithm is used. Otherwise, it overrides the normal algorithm (and the outcome is cached).

  `__weakref__`

    list of weak references to the object (if defined)

  `broadcast`(clients, message)

    Broadcast message to the one or more clients. Use this method if you want to send same message to lots of clients, as it contains several optimizations and will work fast than just having loop in your code.

    clients
      Clients iterable
    message
      Message to send.
  `close`()
  `is_closed`

    Check if connection was closed

  `send`(message, binary=False)

    Send message to the client.

    message
      Message to send.